﻿using System;
using System.ComponentModel.Composition;
using Cubicle.Core.Interfaces;

namespace Cubicle.Core.Processors.Parameter
{
    [Export("Cubicle.Processors.Parameter.EnumStringParameterResolver", typeof(IProcessor<ParameterRequest>))]
    [ExportMetadata("Name", "EnumStringParameterResolver")]
    [ExportMetadata("Description", "Resolves an enum string to a strongly typed Enum.")]
    public class EnumStringParameterResolver : ConditionalProcessor<ParameterRequest>
    {
        private readonly Type _stringType = typeof(string);

        #region Overrides of Processor<ParameterRequest>

        public override bool Condition(ParameterRequest request)
        {
            return request.SourceType == _stringType 
                   && request.TargetType.IsEnum;
        }

        public override ParameterRequest ConditionalProcess(ParameterRequest request)
        {
            var strValue = (string)request.Source;
            var result = Enum.Parse(request.TargetType, strValue);
            request.Complete(result);
            return request;
        }

        #endregion
    }
}