﻿using System;
using System.ComponentModel.Composition;
using Cubicle.Core.Interfaces;

namespace Cubicle.Core.Processors.Parameter
{
    [Export("Cubicle.Processors.Parameter.ValueToJaggedArrayParameterResolver", typeof(IProcessor<ParameterRequest>))]
    [ExportMetadata("Name", "ValueToJaggedArrayParameterResolver")]
    [ExportMetadata("Description", "Resolves a single parameter to a jagged array.")]
    public class ValueToJaggedArrayParameterResolver : ConditionalProcessor<ParameterRequest>
    {
        [Import]
        private ParameterProcessor _parameterResolver;

        public override bool Condition(ParameterRequest request)
        {
            var valueType = request.SourceType;
            return !valueType.IsArray
                   && request.TargetType.IsArray
                   && request.TargetType.GetElementType().IsArray;
        }

        public override ParameterRequest ConditionalProcess(ParameterRequest request)
        {
            // Value will come in as a jagged array
            var arrType = request.TargetType.GetElementType();
            var elemType = arrType.GetElementType();
            var result = Array.CreateInstance(arrType, 1);
            var arr = Array.CreateInstance(elemType, 1);
            result.SetValue(arr, 0);

            var parRequest = new ParameterRequest(elemType, request.Source);
            _parameterResolver.Process(parRequest);
            arr.SetValue(parRequest.Result, 0);

            request.Complete(result);
            return request;
        }

    }
}