﻿using System;
using System.IO;
using System.Linq;
using System.Runtime.Versioning;
using NuGet;

namespace Exocortex.Excel.Nuget
{
    public class WebProjectSystem : PhysicalFileSystem, IProjectSystem, IFileSystem
    {
        private string BinDir = "bin";

        public WebProjectSystem(string root, string binDir) : base(root)
        {
            BinDir = binDir;
        }

        public void AddFrameworkReference(string name)
        {
            throw new NotImplementedException();
        }

        public void AddReference(string referencePath, Stream stream)
        {
            string fileName = Path.GetFileName(referencePath);
            string fullPath = this.GetFullPath(this.GetReferencePath(fileName));
            this.AddFile(fullPath, stream);
        }

        public object GetPropertyValue(string propertyName)
        {
            if ((propertyName != null) && propertyName.Equals("RootNamespace", StringComparison.OrdinalIgnoreCase))
            {
                return string.Empty;
            }
            return null;
        }

        protected virtual string GetReferencePath(string name)
        {
            return Path.Combine(BinDir, name);
        }

        public bool IsSupportedFile(string path)
        {
            return (!path.StartsWith("tools", StringComparison.OrdinalIgnoreCase) && !Path.GetFileName(path).Equals("app.config", StringComparison.OrdinalIgnoreCase));
        }

        public bool ReferenceExists(string name)
        {
            string referencePath = this.GetReferencePath(name);
            return this.FileExists(referencePath);
        }

        public void RemoveReference(string name)
        {
            this.DeleteFile(this.GetReferencePath(name));
            if (!this.GetFiles(BinDir).Any<string>())
            {
                this.DeleteDirectory(BinDir);
            }
        }

        public string ProjectName
        {
            get
            {
                return base.Root;
            }
        }

        public string ResolvePath(string path)
        {
            return Path.Combine(Root, path);
        }

        public FrameworkName TargetFramework
        {
            get
            {
                return VersionUtility.DefaultTargetFramework;
            }
        }
    }
}

